.parent {
    background-color: #edebeb;
    width: 100%;
    height: 150px;
}

textarea {
    width: 100%;
    box-sizing: border-box;
    /* ※これがないと横にはみ出る */
    height: 120px;
    font-size: large;
    outline: none;
    /* ※ブラウザが標準で付加する線を消したいとき */
    resize: none;
}

.box11 {
    /*    padding: 0.5em 1em;    margin: 2em 0;*/
    color: #5d627b;
    background: white;
    border-top: solid 5px #5d627b;
    box-shadow: 0 3px 5px rgba(0, 0, 0, 0.22);
}

.box18 {
    /*padding: 0.2em 0.5em; margin: 2em 0; */
    color: #565656;
    background: #ffeaea;
    /*background-image: url(https://2.bp.blogspot.com/-u7NQvQSgyAY/Ur1HXta5W7I/AAAAAAAAcfE/omW7_szrzao/s800/dog_corgi.png);*/
    background-size: 150%;
    background-repeat: no-repeat;
    background-position: top right;
    background-color: rgba(255, 255, 255, 0.8);
    background-blend-mode: lighten;
    /*box-shadow: 0px 0px 0px 10px #ffeaea;*/
    border: dashed 2px #ffc3c3;
    /*border-radius: 8px;*/
}

.box16 {
    /*
    padding: 0.5em 1em;margin: 2em 0;
    */
    background: -webkit-repeating-linear-gradient(-45deg, #f0f8ff, #f0f8ff 3px, #e9f4ff 3px, #e9f4ff 7px);
    background: repeating-linear-gradient(-45deg, #f0f8ff, #f0f8ff 3px, #e9f4ff 3px, #e9f4ff 7px);
}

.box24 {
    position: relative;
    padding: 0.5em 0.7em;
    margin: 2em 0;
    background: #6f4b3e;
    color: white;
    font-weight: bold;
}

/* 
.box24:after {
    position: absolute;
    content: '';
    top: 100%;
    left: 30px;
    border: 15px solid transparent;
    border-top: 15px solid #6f4b3e;
    width: 0;
    height: 0;
} */