html {
    --button-color: #333;
    --button-text: #AAA;
}

html[theme=dark] {
    --button-color: #AAA;
    --button-text: #333;
}

button {
    background: var(--button-color);
    color: var(--button-text);
    font-size: 16px;
}